#!/usr/bin/env python
import argparse
import re
import os
import glob
import numpy as np
import matplotlib.gridspec as gridspec
import matplotlib.pyplot as plt

# 図の体裁
plt.style.use('classic')
plt_dic = {}
plt_dic['legend.fancybox'] = True
plt_dic['legend.labelspacing'] = 0.3
plt_dic['legend.numpoints'] = 3
plt_dic['figure.figsize'] = [8, 6]
plt_dic['axes.grid'] = True
plt_dic['font.size'] = 12
plt_dic['legend.fontsize'] = 12
plt_dic['axes.labelsize'] = 14
plt_dic['xtick.major.size'] = 5
plt_dic['xtick.minor.size'] = 3
plt_dic['xtick.direction'] = 'in'
plt_dic['savefig.bbox'] = 'tight'
plt_dic['savefig.dpi'] = 150
plt_dic['savefig.transparent'] = True
plt.rcParams.update(plt_dic)

description = """itx file plotting program."""
par = argparse.ArgumentParser(description=description)
par.add_argument('-i', '--itxfile', default=None)
par.add_argument('-v', '--verbose', default=False,
                 action='store_true', help='Display index')
par.add_argument('-p', '--plot', default=True,
                 action='store_false', help='Display graph')
par.add_argument('-o', '--output', default=None, help='Output file')

args = par.parse_args()

if args.itxfile == None:
    itxfile = glob.glob("*.itx")
    if len(itxfile) == 0:
        print("*.int was not found in current directory.")
        exit()
    base, ext = os.path.splitext(itxfile[0])
    args.itxfile = base + ".itx"
try:
    body = open(args.itxfile).read()
    body = re.sub("\*{12}", "         0.0", body)
except IOError:
    print(('cannot open %s', args.itxfile))

data = re.findall("BEGIN\n(.*?)\nEND", body, re.DOTALL)
lines = len(data[0].split("\n"))
xrd = np.array(data[0].split(), dtype=float).reshape((lines, -1))
lines = len(data[1].split("\n"))
pos = np.array(data[1].split(), dtype=float).reshape((lines, -1))
ind = np.array(data[2].split(), dtype=float).reshape((lines, -1))


fig = plt.figure()
fig.subplots_adjust(hspace=0.000)
gs = gridspec.GridSpec(2, 1, height_ratios=[1, 3])
# differential plot
ax = fig.add_subplot(gs[0])
ax.plot(xrd[:, 0], xrd[:, 3])
ax.set_ylabel("difference")
for xlabel_i in ax.axes.get_xticklabels():
    xlabel_i.set_visible(False)
# data
bx = fig.add_subplot(gs[1], sharex=ax)
bx.plot(xrd[:, 0], xrd[:, 1], "o", ms=3, label="obs.")
bx.plot(xrd[:, 0], xrd[:, 2], "-", ms=3, label="cal.")
bx.plot(xrd[:, 0], xrd[:, 4], "-", ms=3, label="background")
bx.legend(loc="best")
bx.set_xlabel("2theta/theta")
bx.set_ylabel("intensity")

# 指数の出力
if args.verbose:
    for i, p in enumerate(pos):
        print(i, (pos[i, 0]), ind[i])
if args.plot:
    plt.show()

if args.output != None:
    o = open(args.output, "w")
    o.write("# %10s " % ("theta"))  # theta
    o.write("%12s " % ("observation"))  # obs.
    o.write("%12s " % ("simulation"))  # sim.
    o.write("%12s " % ("difference"))  # sim.
    o.write("%12s " % ("backgournd"))  # background
    o.write("\n")
    for x in xrd:
        o.write("%12g " % (x[0]))  # theta
        o.write("%12g " % (x[1]))  # obs.
        o.write("%12g " % (x[2]))  # sim.
        o.write("%12g " % (x[3]))  # difference
        o.write("%12g " % (x[4]))  # background
        o.write("\n")
    print(args.output, "was created.")
